/* vim:set ts=2 sw=2 sts=2 et cin: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(f2f71d91-0451-47ec-aaa0-166663a7711a)]
interface nsISimpleTest : nsISupports
{
  /**
   * This interface adds two numbers together and returns the result.
   */
  long add(in long a, in long b); 
};
