/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(5b912111-c10e-498f-a44c-c713c1843007)]
interface nsIDOMSVGRect : nsISupports
{
  attribute float x;
      // raises DOMException on setting
  attribute float y;
      // raises DOMException on setting
  attribute float width;
      // raises DOMException on setting
  attribute float height;
      // raises DOMException on setting
};
