/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * Information on a Certificate Revocation List (CRL)
 * issued by a Aertificate Authority (CA).
 */
[scriptable, uuid(c185d920-4a3e-11d5-ba27-00108303b117)]
interface nsICRLInfo : nsISupports {

  /**
   *  The issuing CA's organization.
   */
  readonly attribute AString organization;

  /**
   *  The issuing CA's organizational unit.
   */
  readonly attribute AString organizationalUnit;

  /**
   *  The time this CRL was created at.
   */
  readonly attribute PRTime  lastUpdate;

  /**
   *  The time the suggested next update for this CRL.
   */
  readonly attribute PRTime  nextUpdate;

  /**
   *  lastUpdate formatted as a human readable string
   *  formatted according to the environment locale.
   */
  readonly attribute AString lastUpdateLocale;

  /**
   *  nextUpdate formatted as a human readable string
   *  formatted according to the environment locale.
   */
  readonly attribute AString nextUpdateLocale;

  /**
   *  The key identifying the CRL in the database.
   */
  readonly attribute AString nameInDb;

  /**
   *  The URL this CRL was last fetched from.
   */
  readonly attribute AUTF8String lastFetchURL;
};

