/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(481f01a5-0bbb-4abf-8623-f3c2fb5642a9)]
interface nsIDOMSVGStringList : nsISupports
{ 
  readonly attribute unsigned long numberOfItems;
  readonly attribute unsigned long length;

  void clear();
          // raises(nsIDOMDOMException);
  DOMString initialize(in DOMString newItem);
                    // raises(nsIDOMDOMException);
  DOMString getItem(in unsigned long index);
                    // raises(nsIDOMDOMException);
  DOMString insertItemBefore(in DOMString newItem, in unsigned long index);
                    // raises(nsIDOMDOMException);
  DOMString replaceItem(in DOMString newItem, in unsigned long index);
                    // raises(nsIDOMDOMException);
  DOMString removeItem(in unsigned long index);
                    // raises(nsIDOMDOMException);
  DOMString appendItem(in DOMString newItem);
                    // raises(nsIDOMDOMException);
};
