/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMXULLabeledControlEl.idl"
interface nsIDOMHTMLInputElement;

[scriptable, uuid(5bab7935-6219-45f8-9f7c-54ac8e9b023b)]
interface nsIDOMXULTextBoxElement : nsIDOMXULControlElement
{
  // inputField may be any type of editable field, such as an
  // HTML <input type="text"> or <textarea>
  readonly attribute nsIDOMNode inputField;

  readonly attribute long textLength;
  attribute long maxLength;
  attribute long size;
  attribute long selectionStart;
  attribute long selectionEnd;

  attribute DOMString value;
  attribute DOMString type;

  void select();
  void setSelectionRange(in long selectionStart,
                         in long selectionEnd);
};

