/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set ts=2 sw=2 sts=2 et cindent: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMHTMLElement.idl"

/**
 * The nsIDOMHTMLSourceElement interface is the interface to a HTML
 * <source> element.
 *
 * For more information on this interface, please see
 * http://www.whatwg.org/specs/web-apps/current-work/#source
 *
 * @status UNDER_DEVELOPMENT
 */

[scriptable, uuid(333faeb7-93dc-439e-a50e-a9df705b8ba6)]
interface nsIDOMHTMLSourceElement : nsIDOMHTMLElement
{
           attribute DOMString src;
           attribute DOMString type;
           attribute DOMString media;
};
