/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "domstubs.idl"
interface nsIContentFrameMessageManager;

// Sole purpose is to be able to identify the concrete class nsTabChild
[uuid(bf1eddf9-731b-4a4b-bd65-9a712a892832)]
interface nsITabChild : nsISupports
{
  readonly attribute nsIContentFrameMessageManager messageManager;
};

