/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMXULSelectCntrlEl.idl"

[scriptable, uuid(16a38e07-405b-42a3-b45d-c78037757c01)]
interface nsIDOMXULMultiSelectControlElement : nsIDOMXULSelectControlElement
{
  attribute DOMString selType;

  attribute nsIDOMXULSelectControlItemElement currentItem;
  attribute long currentIndex;

  readonly attribute nsIDOMNodeList selectedItems;
  
  void addItemToSelection(in nsIDOMXULSelectControlItemElement item);
  void removeItemFromSelection(in nsIDOMXULSelectControlItemElement item);
  void toggleItemSelection(in nsIDOMXULSelectControlItemElement item);

  void selectItem(in nsIDOMXULSelectControlItemElement item);
  void selectItemRange(in nsIDOMXULSelectControlItemElement startItem, in nsIDOMXULSelectControlItemElement item);

  void selectAll();
  void invertSelection();
  void clearSelection();

  // XXX - temporary, pending implementation of scriptable, 
  //       mutable nsIDOMNodeList for selectedItems
  readonly attribute long selectedCount;
  [binaryname(MultiGetSelectedItem)]
  nsIDOMXULSelectControlItemElement getSelectedItem(in long index);
};
