/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMHTMLElement.idl"

/**
 * The nsIDOMHTMLEmbedElement interface is the interface to a [X]HTML
 * embed element.
 *
 * For more information on this interface, please see
 * http://www.whatwg.org/specs/web-apps/current-work/#the-embed-element
 */

[scriptable, uuid(ca0de9c2-e230-4acb-aa6d-65fc0283bf06)]
interface nsIDOMHTMLEmbedElement : nsIDOMHTMLElement
{
           attribute DOMString        align;
           attribute DOMString        height;
           attribute DOMString        name;
           attribute DOMString        src;
           attribute DOMString        type;
           attribute DOMString        width;
};
