/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_CFStateChangeEventBinding_h__
#define mozilla_dom_CFStateChangeEventBinding_h__

#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/DOMJSClass.h"
#include "mozilla/dom/DOMJSProxyHandler.h"

class XPCWrappedNativeScope;

namespace mozilla {
namespace dom {

struct CFStateChangeEventDict : public MainThreadDictionaryBase {
  CFStateChangeEventDict() {}
  bool Init(JSContext* cx, JSObject* scopeObj, const JS::Value& val);
  bool ToObject(JSContext* cx, JSObject* parentObject, JS::Value *vp);

  bool Init(const nsAString& aJSON)
  {
    mozilla::Maybe<JSAutoRequest> ar;
    mozilla::Maybe<JSAutoCompartment> ac;
    jsval json = JSVAL_VOID;
    JSContext* cx = ParseJSON(aJSON, ar, ac, json);
    NS_ENSURE_TRUE(cx, false);
    return Init(cx, nullptr, json);
  }

  int16_t mAction;
  nsString mNumber;
  int16_t mReason;
  int16_t mServiceClass;
  bool mSuccess;
  int16_t mTimeSeconds;
private:
  // Disallow copy-construction
  CFStateChangeEventDict(const CFStateChangeEventDict&) MOZ_DELETE;
  static bool InitIds(JSContext* cx);
  static bool initedIds;
  static jsid action_id;
  static jsid number_id;
  static jsid reason_id;
  static jsid serviceClass_id;
  static jsid success_id;
  static jsid timeSeconds_id;
};
struct CFStateChangeEventDictInitializer : public CFStateChangeEventDict {
  CFStateChangeEventDictInitializer() {
    // Safe to pass a null context if we pass a null value
    Init(nullptr, nullptr, JS::NullValue());
  }
};

} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_CFStateChangeEventBinding_h__
