/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMDOMRequest.idl"

[scriptable, builtinclass, uuid(ae6acad4-3174-44d6-a491-9ff94ab847c2)]
interface nsIDOMMozSmsRequest : nsIDOMDOMRequest
{
};
