/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
interface nsIMobileMessageCallback;
interface nsIDOMBlob;

%{C++
#define RIL_MMSSERVICE_CONTRACTID "@mozilla.org/mms/rilmmsservice;1"
%}

[scriptable, uuid(3ec33286-8559-11e2-9f38-e76b58650568)]
interface nsIMmsService : nsISupports
{
  void send(in jsval parameters /* MmsParameters */,
            in nsIMobileMessageCallback request);
};
