/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMMozMobileMessageManager;

[scriptable, uuid(baefbb1a-7fe4-11e2-abff-cf1a90f9139f)]
interface nsIDOMMozNavigatorMobileMessage : nsISupports
{
  readonly attribute nsIDOMMozMobileMessageManager mozMobileMessage;
};
